@@ -5,12 +5,15 @@
 import asyncio
 import logging
 
+import pamqp.frame
+import pamqp.heartbeat
+import pamqp.specification
+
 from . import channel as amqp_channel
 from . import constants as amqp_constants
 from . import frame as amqp_frame
 from . import exceptions
 from . import version
-from .compat import ensure_future
 
 
 logger = logging.getLogger(__name__)
@@ -22,14 +25,12 @@ CONNECTING, OPEN, CLOSING, CLOSED = range(4)
 class _StreamWriter(asyncio.StreamWriter):
 
     def write(self, data):
-        ret = super().write(data)
+        super().write(data)
         self._protocol._heartbeat_timer_send_reset()
-        return ret
 
     def writelines(self, data):
-        ret = super().writelines(data)
+        super().writelines(data)
         self._protocol._heartbeat_timer_send_reset()
-        return ret
 
     def write_eof(self):
         ret = super().write_eof()
@@ -64,7 +65,8 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
             client_properties: dict, client-props to tune the client identification
         """
         self._loop = kwargs.get('loop') or asyncio.get_event_loop()
-        super().__init__(asyncio.StreamReader(loop=self._loop), loop=self._loop)
+        self._reader = asyncio.StreamReader(loop=self._loop)
+        super().__init__(self._reader, loop=self._loop)
         self._on_error_callback = kwargs.get('on_error')
 
         self.client_properties = kwargs.get('client_properties', {})
@@ -110,7 +112,8 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
         return False
 
     def connection_lost(self, exc):
-        logger.warning("Connection lost exc=%r", exc)
+        if exc is not None:
+            logger.warning("Connection lost exc=%r", exc)
         self.connection_closed.set()
         self.state = CLOSED
         self._close_channels(exception=exc)
@@ -121,8 +124,7 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
         self._heartbeat_timer_recv_reset()
         super().data_received(data)
 
-    @asyncio.coroutine
-    def ensure_open(self):
+    async def ensure_open(self):
         # Raise a suitable exception if the connection isn't open.
         # Handle cases from the most common to the least common.
 
@@ -134,74 +136,64 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
 
         # If the closing handshake is in progress, let it complete.
         if self.state == CLOSING:
-            yield from self.wait_closed()
+            await self.wait_closed()
             raise exceptions.AmqpClosedConnection()
 
         # Control may only reach this point in buggy third-party subclasses.
         assert self.state == CONNECTING
         raise exceptions.AioamqpException("connection isn't established yet.")
 
-    @asyncio.coroutine
-    def _drain(self):
-        with (yield from self._drain_lock):
+    async def _drain(self):
+        async with self._drain_lock:
             # drain() cannot be called concurrently by multiple coroutines:
             # http://bugs.python.org/issue29930. Remove this lock when no
             # version of Python where this bugs exists is supported anymore.
-            yield from self._stream_writer.drain()
+            await self._stream_writer.drain()
 
-    @asyncio.coroutine
-    def _write_frame(self, frame, request, drain=True):
-        frame.write_frame(request)
+    async def _write_frame(self, channel_id, request, drain=True):
+        amqp_frame.write(self._stream_writer, channel_id, request)
         if drain:
-            yield from self._drain()
+            await self._drain()
 
-    @asyncio.coroutine
-    def close(self, no_wait=False, timeout=None):
+    async def close(self, no_wait=False, timeout=None):
         """Close connection (and all channels)"""
-        yield from self.ensure_open()
+        await self.ensure_open()
         self.state = CLOSING
-        frame = amqp_frame.AmqpRequest(self._stream_writer, amqp_constants.TYPE_METHOD, 0)
-        frame.declare_method(
-            amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_CLOSE)
-        encoder = amqp_frame.AmqpEncoder()
-        # we request a clean connection close
-        encoder.write_short(0)
-        encoder.write_shortstr('')
-        encoder.write_short(0)
-        encoder.write_short(0)
-        yield from self._write_frame(frame, encoder)
-        if not no_wait:
-            yield from self.wait_closed(timeout=timeout)
+        request = pamqp.specification.Connection.Close(
+            reply_code=0,
+            reply_text='',
+            class_id=0,
+            method_id=0
+        )
 
-    @asyncio.coroutine
-    def wait_closed(self, timeout=None):
-        yield from asyncio.wait_for(self.connection_closed.wait(), timeout=timeout, loop=self._loop)
+        await self._write_frame(0, request)
+        if not no_wait:
+            await self.wait_closed(timeout=timeout)
+
+    async def wait_closed(self, timeout=None):
+        await asyncio.wait_for(self.connection_closed.wait(), timeout=timeout, loop=self._loop)
         if self._heartbeat_worker is not None:
             try:
-                yield from asyncio.wait_for(self._heartbeat_worker, timeout=timeout, loop=self._loop)
+                await asyncio.wait_for(self._heartbeat_worker, timeout=timeout, loop=self._loop)
             except asyncio.CancelledError:
                 pass
 
-    @asyncio.coroutine
-    def close_ok(self, frame):
-        logger.info("Recv close ok")
+    async def close_ok(self, frame):
         self._stream_writer.close()
 
-    @asyncio.coroutine
-    def start_connection(self, host, port, login, password, virtualhost, ssl=False,
-            login_method='AMQPLAIN', insist=False):
+    async def start_connection(self, host, port, login, password, virtualhost, ssl=False,
+            login_method='PLAIN', insist=False):
         """Initiate a connection at the protocol level
             We send `PROTOCOL_HEADER'
         """
 
-        if login_method != 'AMQPLAIN':
-            # TODO
-            logger.warning('only AMQPLAIN login_method is supported, falling back to AMQPLAIN')
+        if login_method != 'PLAIN':
+            logger.warning('only PLAIN login_method is supported, falling back to AMQPLAIN')
 
         self._stream_writer.write(amqp_constants.PROTOCOL_HEADER)
 
         # Wait 'start' method from the server
-        yield from self.dispatch_frame()
+        await self.dispatch_frame()
 
         client_properties = {
             'capabilities': {
@@ -219,10 +211,10 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
         }
 
         # waiting reply start with credentions and co
-        yield from self.start_ok(client_properties, 'AMQPLAIN', auth, self.server_locales[0])
+        await self.start_ok(client_properties, 'PLAIN', auth, self.server_locales)
 
         # wait for a "tune" reponse
-        yield from self.dispatch_frame()
+        await self.dispatch_frame()
 
         tune_ok = {
             'channel_max': self.connection_tunning.get('channel_max', self.server_channel_max),
@@ -230,7 +222,7 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
             'heartbeat': self.connection_tunning.get('heartbeat', self.server_heartbeat),
         }
         # "tune" the connexion with max channel, max frame, heartbeat
-        yield from self.tune_ok(**tune_ok)
+        await self.tune_ok(**tune_ok)
 
         # update connection tunning values
         self.server_frame_max = tune_ok['frame_max']
@@ -242,58 +234,50 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
             self._heartbeat_timer_send_reset()
 
         # open a virtualhost
-        yield from self.open(virtualhost, capabilities='', insist=insist)
+        await self.open(virtualhost, capabilities='', insist=insist)
 
         # wait for open-ok
-        frame = yield from self.get_frame()
-        yield from self.dispatch_frame(frame)
-        if (frame.frame_type == amqp_constants.TYPE_METHOD and
-                frame.class_id == amqp_constants.CLASS_CONNECTION and
-                frame.method_id == amqp_constants.CONNECTION_CLOSE):
-            raise exceptions.AmqpClosedConnection()
+        channel, frame = await self.get_frame()
+        await self.dispatch_frame(channel, frame)
 
+        await self.ensure_open()
         # for now, we read server's responses asynchronously
-        self.worker = ensure_future(self.run(), loop=self._loop)
+        self.worker = asyncio.ensure_future(self.run(), loop=self._loop)
 
-    @asyncio.coroutine
-    def get_frame(self):
+    async def get_frame(self):
         """Read the frame, and only decode its header
 
         """
-        frame = amqp_frame.AmqpResponse(self._stream_reader)
-        yield from frame.read_frame()
+        return await amqp_frame.read(self._stream_reader)
 
-        return frame
-
-    @asyncio.coroutine
-    def dispatch_frame(self, frame=None):
+    async def dispatch_frame(self, frame_channel=None, frame=None):
         """Dispatch the received frame to the corresponding handler"""
 
         method_dispatch = {
-            (amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_CLOSE): self.server_close,
-            (amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_CLOSE_OK): self.close_ok,
-            (amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_TUNE): self.tune,
-            (amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_START): self.start,
-            (amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_OPEN_OK): self.open_ok,
+            pamqp.specification.Connection.Close.name: self.server_close,
+            pamqp.specification.Connection.CloseOk.name: self.close_ok,
+            pamqp.specification.Connection.Tune.name: self.tune,
+            pamqp.specification.Connection.Start.name: self.start,
+            pamqp.specification.Connection.OpenOk.name: self.open_ok,
         }
-        if not frame:
-            frame = yield from self.get_frame()
+        if frame_channel is None and frame is None:
+            frame_channel, frame = await self.get_frame()
 
-        if frame.frame_type == amqp_constants.TYPE_HEARTBEAT:
+        if isinstance(frame, pamqp.heartbeat.Heartbeat):
             return
 
-        if frame.channel is not 0:
-            channel = self.channels.get(frame.channel)
+        if frame_channel != 0:
+            channel = self.channels.get(frame_channel)
             if channel is not None:
-                yield from channel.dispatch_frame(frame)
+                await channel.dispatch_frame(frame)
             else:
-                logger.info("Unknown channel %s", frame.channel)
+                logger.info("Unknown channel %s", frame_channel)
             return
 
-        if (frame.class_id, frame.method_id) not in method_dispatch:
-            logger.info("frame %s %s is not handled", frame.class_id, frame.method_id)
+        if frame.name not in method_dispatch:
+            logger.info("frame %s is not handled", frame.name)
             return
-        yield from method_dispatch[(frame.class_id, frame.method_id)](frame)
+        await method_dispatch[frame.name](frame)
 
     def release_channel_id(self, channel_id):
         """Called from the channel instance, it relase a previously used
@@ -319,18 +303,17 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
 
         if self._on_error_callback:
             if asyncio.iscoroutinefunction(self._on_error_callback):
-                ensure_future(self._on_error_callback(exception), loop=self._loop)
+                asyncio.ensure_future(self._on_error_callback(exception), loop=self._loop)
             else:
                 self._on_error_callback(exceptions.ChannelClosed(exception))
 
         for channel in self.channels.values():
             channel.connection_closed(reply_code, reply_text, exception)
 
-    @asyncio.coroutine
-    def run(self):
+    async def run(self):
         while not self.stop_now.done():
             try:
-                yield from self.dispatch_frame()
+                await self.dispatch_frame()
             except exceptions.AmqpClosedConnection as exc:
                 logger.info("Close connection")
                 self.stop_now.set_result(None)
@@ -339,25 +322,22 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
             except Exception:  # pylint: disable=broad-except
                 logger.exception('error on dispatch')
 
-    @asyncio.coroutine
-    def heartbeat(self):
+    async def heartbeat(self):
         """ deprecated heartbeat coroutine
 
         This coroutine is now a no-op as the heartbeat is handled directly by
         the rest of the AmqpProtocol class.  This is kept around for backwards
         compatibility purposes only.
         """
-        yield from self.stop_now
+        await self.stop_now
 
-    @asyncio.coroutine
-    def send_heartbeat(self):
+    async def send_heartbeat(self):
         """Sends an heartbeat message.
         It can be an ack for the server or the client willing to check for the
         connexion timeout
         """
-        frame = amqp_frame.AmqpRequest(self._stream_writer, amqp_constants.TYPE_HEARTBEAT, 0)
-        request = amqp_frame.AmqpEncoder()
-        yield from self._write_frame(frame, request)
+        request = pamqp.heartbeat.Heartbeat()
+        await self._write_frame(0, request)
 
     def _heartbeat_timer_recv_timeout(self):
         # 4.2.7 If a peer detects no incoming traffic (i.e. received octets) for
@@ -385,7 +365,7 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
             self.server_heartbeat,
             self._heartbeat_trigger_send.set)
         if self._heartbeat_worker is None:
-            self._heartbeat_worker = ensure_future(self._heartbeat(), loop=self._loop)
+            self._heartbeat_worker = asyncio.ensure_future(self._heartbeat(), loop=self._loop)
 
     def _heartbeat_stop(self):
         self.server_heartbeat = None
@@ -396,97 +376,72 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
         if self._heartbeat_worker is not None:
             self._heartbeat_worker.cancel()
 
-    @asyncio.coroutine
-    def _heartbeat(self):
+    async def _heartbeat(self):
         while self.state != CLOSED:
-            yield from self._heartbeat_trigger_send.wait()
+            await self._heartbeat_trigger_send.wait()
             self._heartbeat_trigger_send.clear()
-            yield from self.send_heartbeat()
+            await self.send_heartbeat()
 
     # Amqp specific methods
-    @asyncio.coroutine
-    def start(self, frame):
+    async def start(self, frame):
         """Method sent from the server to begin a new connection"""
-        response = amqp_frame.AmqpDecoder(frame.payload)
+        self.version_major = frame.version_major
+        self.version_minor = frame.version_minor
+        self.server_properties = frame.server_properties
+        self.server_mechanisms = frame.mechanisms
+        self.server_locales = frame.locales
 
-        self.version_major = response.read_octet()
-        self.version_minor = response.read_octet()
-        self.server_properties = response.read_table()
-        self.server_mechanisms = response.read_longstr().split(' ')
-        self.server_locales = response.read_longstr().split(' ')
+    async def start_ok(self, client_properties, mechanism, auth, locale):
+        def credentials():
+            return '\0{LOGIN}\0{PASSWORD}'.format(**auth)
 
-    @asyncio.coroutine
-    def start_ok(self, client_properties, mechanism, auth, locale):
-        frame = amqp_frame.AmqpRequest(self._stream_writer, amqp_constants.TYPE_METHOD, 0)
-        frame.declare_method(
-            amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_START_OK)
-        request = amqp_frame.AmqpEncoder()
-        request.write_table(client_properties)
-        request.write_shortstr(mechanism)
-        request.write_table(auth)
-        request.write_shortstr(locale.encode())
-        yield from self._write_frame(frame, request)
+        request = pamqp.specification.Connection.StartOk(
+            client_properties=client_properties,
+            mechanism=mechanism,
+            locale=locale,
+            response=credentials()
+        )
+        await self._write_frame(0, request)
 
-    @asyncio.coroutine
-    def server_close(self, frame):
+    async def server_close(self, frame):
         """The server is closing the connection"""
         self.state = CLOSING
-        response = amqp_frame.AmqpDecoder(frame.payload)
-        reply_code = response.read_short()
-        reply_text = response.read_shortstr()
-        class_id = response.read_short()
-        method_id = response.read_short()
+        reply_code = frame.reply_code
+        reply_text = frame.reply_text
+        class_id = frame.class_id
+        method_id = frame.method_id
         logger.warning("Server closed connection: %s, code=%s, class_id=%s, method_id=%s",
             reply_text, reply_code, class_id, method_id)
         self._close_channels(reply_code, reply_text)
-        self._close_ok()
+        await self._close_ok()
         self._stream_writer.close()
 
-    def _close_ok(self):
-        frame = amqp_frame.AmqpRequest(self._stream_writer, amqp_constants.TYPE_METHOD, 0)
-        frame.declare_method(
-            amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_CLOSE_OK)
-        request = amqp_frame.AmqpEncoder()
-        yield from self._write_frame(frame, request)
+    async def _close_ok(self):
+        request = pamqp.specification.Connection.CloseOk()
+        await self._write_frame(0, request)
 
-    @asyncio.coroutine
-    def tune(self, frame):
-        decoder = amqp_frame.AmqpDecoder(frame.payload)
-        self.server_channel_max = decoder.read_short()
-        self.server_frame_max = decoder.read_long()
-        self.server_heartbeat = decoder.read_short()
+    async def tune(self, frame):
+        self.server_channel_max = frame.channel_max
+        self.server_frame_max = frame.frame_max
+        self.server_heartbeat = frame.heartbeat
 
-    @asyncio.coroutine
-    def tune_ok(self, channel_max, frame_max, heartbeat):
-        frame = amqp_frame.AmqpRequest(self._stream_writer, amqp_constants.TYPE_METHOD, 0)
-        frame.declare_method(
-            amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_TUNE_OK)
-        encoder = amqp_frame.AmqpEncoder()
-        encoder.write_short(channel_max)
-        encoder.write_long(frame_max)
-        encoder.write_short(heartbeat)
+    async def tune_ok(self, channel_max, frame_max, heartbeat):
+        request = pamqp.specification.Connection.TuneOk(
+            channel_max, frame_max, heartbeat
+        )
+        await self._write_frame(0, request)
 
-        yield from self._write_frame(frame, encoder)
-
-    @asyncio.coroutine
-    def secure_ok(self, login_response):
+    async def secure_ok(self, login_response):
         pass
 
-    @asyncio.coroutine
-    def open(self, virtual_host, capabilities='', insist=False):
+    async def open(self, virtual_host, capabilities='', insist=False):
         """Open connection to virtual host."""
-        frame = amqp_frame.AmqpRequest(self._stream_writer, amqp_constants.TYPE_METHOD, 0)
-        frame.declare_method(
-            amqp_constants.CLASS_CONNECTION, amqp_constants.CONNECTION_OPEN)
-        encoder = amqp_frame.AmqpEncoder()
-        encoder.write_shortstr(virtual_host)
-        encoder.write_shortstr(capabilities)
-        encoder.write_bool(insist)
+        request = pamqp.specification.Connection.Open(
+            virtual_host, capabilities, insist
+        )
+        await self._write_frame(0, request)
 
-        yield from self._write_frame(frame, encoder)
-
-    @asyncio.coroutine
-    def open_ok(self, frame):
+    async def open_ok(self, frame):
         self.state = OPEN
         logger.info("Recv open ok")
 
@@ -494,12 +449,11 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
     ## aioamqp public methods
     #
 
-    @asyncio.coroutine
-    def channel(self, **kwargs):
+    async def channel(self, **kwargs):
         """Factory to create a new channel
 
         """
-        yield from self.ensure_open()
+        await self.ensure_open()
         try:
             channel_id = self.channels_ids_free.pop()
         except KeyError:
@@ -511,5 +465,5 @@ class AmqpProtocol(asyncio.StreamReaderProtocol):
             channel_id = self.channels_ids_ceil
         channel = self.CHANNEL_FACTORY(self, channel_id, **kwargs)
         self.channels[channel_id] = channel
-        yield from channel.open()
+        await channel.open()
         return channel
