@@ -1,30 +1,28 @@
 """Aioamqp tests"""
 
-import unittest
+import asynctest
 import socket
 
 from aioamqp import connect
 from aioamqp.protocol import OPEN
 
-from . import testing, testcase
+from . import testcase
 
 
-class AmqpConnectionTestCase(testcase.RabbitTestCase, unittest.TestCase):
+class AmqpConnectionTestCase(testcase.RabbitTestCaseMixin, asynctest.TestCase):
 
-    @testing.coroutine
-    def test_connect(self):
-        _transport, proto = yield from connect(host=self.host, port=self.port, virtualhost=self.vhost, loop=self.loop)
+    async def test_connect(self):
+        _transport, proto = await connect(host=self.host, port=self.port, virtualhost=self.vhost, loop=self.loop)
         self.assertEqual(proto.state, OPEN)
         self.assertIsNotNone(proto.server_properties)
-        yield from proto.close()
+        await proto.close()
 
-    @testing.coroutine
-    def test_connect_tuning(self):
+    async def test_connect_tuning(self):
         # frame_max should be higher than 131072
         frame_max = 131072
         channel_max = 10
         heartbeat = 100
-        _transport, proto = yield from connect(
+        _transport, proto = await connect(
             host=self.host,
             port=self.port,
             virtualhost=self.vhost,
@@ -46,12 +44,11 @@ class AmqpConnectionTestCase(testcase.RabbitTestCase, unittest.TestCase):
         self.assertEqual(proto.server_frame_max, frame_max)
         self.assertEqual(proto.server_heartbeat, heartbeat)
 
-        yield from proto.close()
+        await proto.close()
 
-    @testing.coroutine
-    def test_socket_nodelay(self):
-        transport, proto = yield from connect(host=self.host, port=self.port, virtualhost=self.vhost, loop=self.loop)
+    async def test_socket_nodelay(self):
+        transport, proto = await connect(host=self.host, port=self.port, virtualhost=self.vhost, loop=self.loop)
         sock = transport.get_extra_info('socket')
         opt_val = sock.getsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY)
         self.assertNotEqual(opt_val, 0)
-        yield from proto.close()
+        await proto.close()
