@@ -1,19 +1,17 @@
-import unittest
-import unittest.mock
+import asynctest
+import asynctest.mock
 import asyncio
 
 from aioamqp.protocol import OPEN, CLOSED
 
 from . import testcase
-from . import testing
 
 
-class ConnectionLostTestCase(testcase.RabbitTestCase, unittest.TestCase):
+class ConnectionLostTestCase(testcase.RabbitTestCaseMixin, asynctest.TestCase):
 
     _multiprocess_can_split_ = True
 
-    @testing.coroutine
-    def test_connection_lost(self):
+    async def test_connection_lost(self):
 
         self.callback_called = False
 
@@ -26,7 +24,7 @@ class ConnectionLostTestCase(testcase.RabbitTestCase, unittest.TestCase):
         self.assertEqual(amqp.state, OPEN)
         self.assertTrue(channel.is_open)
         amqp._stream_reader._transport.close()  # this should have the same effect as the tcp connection being lost
-        yield from asyncio.wait_for(amqp.worker, 1, loop=self.loop)
+        await asyncio.wait_for(amqp.worker, 1, loop=self.loop)
         self.assertEqual(amqp.state, CLOSED)
         self.assertFalse(channel.is_open)
         self.assertTrue(self.callback_called)
