@@ -6,22 +6,21 @@ import aioamqp
 import sys
 
 
-@asyncio.coroutine
-def new_task():
+async def new_task():
     try:
-        transport, protocol = yield from aioamqp.connect('localhost', 5672)
+        transport, protocol = await aioamqp.connect('localhost', 5672)
     except aioamqp.AmqpClosedConnection:
         print("closed connections")
         return
 
 
-    channel = yield from protocol.channel()
+    channel = await protocol.channel()
 
-    yield from channel.queue('task_queue', durable=True)
+    await channel.queue('task_queue', durable=True)
 
     message = ' '.join(sys.argv[1:]) or "Hello World!"
 
-    yield from channel.basic_publish(
+    await channel.basic_publish(
         payload=message,
         exchange_name='',
         routing_key='task_queue',
@@ -31,10 +30,8 @@ def new_task():
     )
     print(" [x] Sent %r" % message,)
 
-    yield from protocol.close()
+    await protocol.close()
     transport.close()
 
 
 asyncio.get_event_loop().run_until_complete(new_task())
-
-
